#!/usr/bin/env python

#****************************************************************************************************************************************************
# Copyright (c) 2014 Freescale Semiconductor, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright notice,
#      this list of conditions and the following disclaimer.
#
#    * Redistributions in binary form must reproduce the above copyright notice,
#      this list of conditions and the following disclaimer in the documentation
#      and/or other materials provided with the distribution.
#
#    * Neither the name of the Freescale Semiconductor, Inc. nor the names of
#      its contributors may be used to endorse or promote products derived from
#      this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#****************************************************************************************************************************************************

from FslBuildGen.DataTypes import *
from FslBuildGen.Exceptions import *
from FslBuildGen import IOUtil

def LocateToplevelPackage(packages):
    for package in packages:
        if package.Type == PackageType.TopLevel:
            return package
    return None

def ToIfString(entries):
    res = []
    for entry in entries:
        res.append('[ "$FSL_PLATFORM_NAME" != "%s" ]' % entry.Name)
    return res

def GeneratePlatformCheckString(entries):
    return " && ".join(ToIfString(entries))

def IsOnList(pathIgnoreList, entry):
    if not pathIgnoreList:
        return False
    packagePath = entry.lower()
    for entry in pathIgnoreList:
        if packagePath.find(entry) >= 0:
            return True
    return False

def GenerateBuildScriptAllWithFilter(config, packages, pathFilter, pathIgnoreList, pathWhiteList, dstFilename):
    template = IOUtil.ReadFile(IOUtil.Join(config.SDKConfigTemplatePath, "build.sh"))
    sdkPath = config.SDKPath + '/'
    str = ""

    topLevelPackage = LocateToplevelPackage(packages)
    for package in topLevelPackage.ResolvedBuildOrder:
        if package.Type == PackageType.Library and (pathFilter == None or IsOnList(pathWhiteList, package.AbsolutePath) or not IsOnList(pathIgnoreList, package.AbsolutePath) ):
            str += "pushd " + config.ToBashPath(package.AbsolutePath) + " > /dev/null\n"
            str += 'make "$@"\n'
            str += "popd > /dev/null\n"
        if package.Type == PackageType.Executable and (pathFilter == None or package.AbsolutePath.lower().find(pathFilter) >= 0):
            unsupportedPlatformList = package.GetUnsupportedPlatformList()
            if len(unsupportedPlatformList) > 0:
                str += 'if %s; then\n' % GeneratePlatformCheckString(unsupportedPlatformList)
                str += "  pushd " + config.ToBashPath(package.AbsolutePath) + " > /dev/null\n"
                str += '  make "$@"\n'
                str += "  popd > /dev/null\n"
                str += "fi\n"
            else:
                str += "pushd " + config.ToBashPath(package.AbsolutePath) + " > /dev/null\n"
                str += 'make "$@"\n'
                str += "popd > /dev/null\n"
    build = template
    build = build.replace("##PACKAGE_BUILD_COMMANDS##", str)
    dstFile = IOUtil.Join(sdkPath, dstFilename)
    if not config.DisableWrite:
        IOUtil.WriteFileIfChanged(dstFile, build)


def GenerateBuildScriptAll(config, packages):
    if not config.IsSDKBuild:
        return
    # FIX: hardcoded filtering
    GenerateBuildScriptAllWithFilter(config, packages, None, None, None, "build.sh")
    GenerateBuildScriptAllWithFilter(config, packages, "gles2", ["gles3", "openvg", "fsldemoappvg", "fslgraphicsvg", "fslnativegraphicsvg", "g2d", "stub"], None, "build_GLES2.sh")
    GenerateBuildScriptAllWithFilter(config, packages, "gles3", ["gles2", "openvg", "fsldemoappvg", "fslgraphicsvg", "fslnativegraphicsvg", "g2d", "stub"], None, "build_GLES3.sh")
    GenerateBuildScriptAllWithFilter(config, packages, "openvg", ["gles2", "gles3", "g2d", "stub"], None, "build_OpenVG.sh")
    GenerateBuildScriptAllWithFilter(config, packages, "g2d", ["gles2", "gles3", "openvg", "fsldemoappvg", "fslgraphicsvg", "fslnativegraphicsvg", "egl"], None, "build_G2D.sh")
