#!/usr/bin/env python

#****************************************************************************************************************************************************
# Copyright (c) 2014 Freescale Semiconductor, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright notice,
#      this list of conditions and the following disclaimer.
#
#    * Redistributions in binary form must reproduce the above copyright notice,
#      this list of conditions and the following disclaimer in the documentation
#      and/or other materials provided with the distribution.
#
#    * Neither the name of the Freescale Semiconductor, Inc. nor the names of
#      its contributors may be used to endorse or promote products derived from
#      this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#****************************************************************************************************************************************************

import itertools
import os
import os.path
from VariantHelper import VariantHelper
from FslBuildGen import IOUtil, MakeFileHelper, Util
from FslBuildGen.DataTypes import *
from FslBuildGen.Exceptions import *

class CodeTemplateVC(object):
    def __init__(self, config, filePrefix, hasManifest):
        super(CodeTemplateVC, self).__init__()
        if hasManifest:
            self.Manifest = IOUtil.ReadFile(IOUtil.Join(config.SDKConfigTemplatePath, "GenVC2013/%s.manifest" % filePrefix))
        else:
            self.Manifest = ""
        self.TemplateSLN = IOUtil.ReadFile(IOUtil.Join(config.SDKConfigTemplatePath, "GenVC2013/%s.sln" % filePrefix))
        self.SLNSnippet1 = IOUtil.ReadFile(IOUtil.Join(config.SDKConfigTemplatePath, "GenVC2013/%sSnippet1.sln" % filePrefix))
        self.SLNSnippet2 = IOUtil.ReadFile(IOUtil.Join(config.SDKConfigTemplatePath, "GenVC2013/%sSnippet2.sln" % filePrefix))
        self.Master = IOUtil.ReadFile(IOUtil.Join(config.SDKConfigTemplatePath, "GenVC2013/%sMaster.vcxproj" % filePrefix))
        self.Snippet1 = IOUtil.ReadFile(IOUtil.Join(config.SDKConfigTemplatePath, "GenVC2013/%sSnippet1.vcxproj" % filePrefix))
        self.Snippet2 = IOUtil.ReadFile(IOUtil.Join(config.SDKConfigTemplatePath, "GenVC2013/%sSnippet2.vcxproj" % filePrefix))
        self.Snippet2_1 = IOUtil.ReadFile(IOUtil.Join(config.SDKConfigTemplatePath, "GenVC2013/%sSnippet2_1.vcxproj" % filePrefix))
        self.Snippet3 = IOUtil.ReadFile(IOUtil.Join(config.SDKConfigTemplatePath, "GenVC2013/%sSnippet3.vcxproj" % filePrefix))
        self.Snippet4 = IOUtil.ReadFile(IOUtil.Join(config.SDKConfigTemplatePath, "GenVC2013/%sSnippet4.vcxproj" % filePrefix))
        self.Snippet5 = IOUtil.ReadFile(IOUtil.Join(config.SDKConfigTemplatePath, "GenVC2013/%sSnippet5.vcxproj" % filePrefix))
        self.Snippet6 = IOUtil.ReadFile(IOUtil.Join(config.SDKConfigTemplatePath, "GenVC2013/%sSnippet6.vcxproj" % filePrefix))
        self.Snippet7 = IOUtil.ReadFile(IOUtil.Join(config.SDKConfigTemplatePath, "GenVC2013/%sSnippet7.vcxproj" % filePrefix))
        self.Snippet8 = IOUtil.ReadFile(IOUtil.Join(config.SDKConfigTemplatePath, "GenVC2013/%sSnippet8.vcxproj" % filePrefix))


class GeneratorVC(object):
    def __init__(self, config, packages, platformName):
        super(GeneratorVC, self).__init__()
        libTemplate = CodeTemplateVC(config, "Lib", False)
        exeTemplate = CodeTemplateVC(config, "Exe", True)
        for package in packages:
            if package.Type == PackageType.Library:
                self.__GenerateLibraryBuildFile(config, package, platformName, libTemplate)
            elif package.Type == PackageType.Executable:
                self.__GenerateLibraryBuildFile(config, package, platformName, exeTemplate)

    def __GenerateLibraryBuildFile(self, config, package, platformName, template):
        platformProjectId = 'ErrorNotDefined'
        if package.ResolvedPlatform == None or package.ResolvedPlatform.ProjectId == None:
            raise XmlFormatException("Missing project id for windows platform")

        platformProjectId = package.ResolvedPlatform.ProjectId
        variantHelper = VariantHelper(package)
        libName = package.ShortName

        slnSnippet1 = self.__GenerateSLNPackageVariantConfig(variantHelper, template.SLNSnippet1, package)
        slnSnippet2 = self.__GenerateSLNPackageVariants(variantHelper, template.SLNSnippet2)
        libDep1SLN = self.__GenerateSLNDependencies1(package)
        libDep2SLN = self.__GenerateSLNDependencies2(config, package)
        libDep3SLN = self.__GenerateSLNDependencies3(variantHelper, template.SLNSnippet2, package)

        build = template.TemplateSLN
        build = build.replace("##PACKAGE_TARGET_NAME##", libName)
        build = build.replace("##PACKAGE_PLATFORM_PROJECT_ID##", platformProjectId)
        build = build.replace("##PACKAGE_LIBRARY_DEPENDENCIES1##", libDep1SLN)
        build = build.replace("##PACKAGE_LIBRARY_DEPENDENCIES2##", libDep2SLN)
        build = build.replace("##PACKAGE_LIBRARY_DEPENDENCIES3##", libDep3SLN)
        build = build.replace("##SNIPPET1##", slnSnippet1)
        build = build.replace("##SNIPPET2##", slnSnippet2)
        buildSLN = build

        projectionConfigurations = self.__GenerateProjectConfigurations(variantHelper, template.Snippet1, package)

        libDepVC = self.__GenerateVCDependencies(template.Snippet2_1, config, package)
        if len(libDepVC) > 0:
            libDepVC = template.Snippet2.replace("##SNIPPET##", libDepVC)

        includeFiles = self.__CreateVisualStudioStyleFileList(template.Snippet3, package.ResolvedBuildAllIncludeFiles)
        sourceFiles = self.__CreateVisualStudioStyleFileList(template.Snippet4, package.ResolvedBuildSourceFiles)

        snippet5 = self.__VariantSimpleReplace(variantHelper, template.Snippet5, package)
        importGroups = self.__GenerateImportGroups(variantHelper, template.Snippet6, package)
        variantPropertyGroups = self.__GenerateVariantPropertyGroups(variantHelper, template.Snippet7, package)
        compilerSettingsGroups = self.__GenerateCompilerSettingsGroups(variantHelper, template.Snippet8, package)

        build = template.Master
        build = build.replace("##SNIPPET1##", projectionConfigurations)
        build = build.replace("##SNIPPET2##", libDepVC)
        build = build.replace("##SNIPPET3##", includeFiles)
        build = build.replace("##SNIPPET4##", sourceFiles)
        build = build.replace("##SNIPPET5##", snippet5)
        build = build.replace("##SNIPPET6##", importGroups)
        build = build.replace("##SNIPPET7##", variantPropertyGroups)
        build = build.replace("##SNIPPET8##", compilerSettingsGroups)
        build = build.replace("##PACKAGE_PLATFORM_PROJECT_ID##", platformProjectId)
        build = build.replace("##PACKAGE_TARGET_NAME##", libName)

        #build = build.replace("##VARIANT##", "")
        buildVC = build

        dstName = package.ShortName
        if (package.ResolvedPlatform == None or package.ResolvedPlatform.ProjectId == None):
            raise InternalErrorException("Could not find project name");
            #dstName = 'test'
        #dstName = 'test'
        dstFileSLN = IOUtil.Join(package.AbsolutePath, dstName + ".sln")
        dstFileVC = IOUtil.Join(package.AbsolutePath, dstName + ".vcxproj")
        if not config.DisableWrite:
            IOUtil.WriteFileIfChanged(dstFileSLN, buildSLN)
            IOUtil.WriteFileIfChanged(dstFileVC, buildVC)

        if package.Type == PackageType.Executable:
            manifestFile = IOUtil.Join(package.AbsolutePath, dstName + ".manifest")
            IOUtil.WriteFileIfChanged(manifestFile, template.Manifest)


    def __GetExternalLibraryDependencies(self, package):
        libPaths = []
        buildOrder = list(package.ResolvedBuildOrder)
        buildOrder.reverse()
        for entry in buildOrder:
            externalList = Util.FilterByType(entry.ResolvedDirectExternalDependencies, ExternalDependencyType.StaticLib)
            libPaths += externalList
        return libPaths


    def __CreateVisualStudioStyleFileList(self, snippet, entries):
        if entries == None:
            return ""
        res = []
        for entry in entries:
            str = snippet.replace("##FILE_PATH##", entry.replace('/', '\\'))
            res.append(str)
        return "\n".join(res)


    def __CreateVisualStudioStyleList(self, entries):
        if entries != None and len(entries) > 0:
            return ";".join(entries).replace("/", "\\")
        else:
            return ""


    def __CreateVisualStudioStyleDefineList(self, entries):
        if entries != None and len(entries) > 0:
            return ";".join(entries).replace("/", "\\") + ';'
        else:
            return ""


    def __GenerateVariantPostfix(self, package):
        variants = package.ResolvedAllVariantDict.keys()
        if len(variants) > 0:
            return "_" + "_".join(variants)
        else:
            return ""


    def FilterVariants(self, variants):
        dict = {}
        for entry in variants:
            if not dict.has_key(entry.Type):
                dict[entry.Type] = []
            dict[entry.Type].append(entry)
        return dict


    def __BuildConfigurationName(self,entry):
        return "-".join(entry)

    def __ContainsVariant(self, list, variantName):
        for entry in list:
            if entry.Name == variantName:
                return True
        return False


    def __BuildMatchedVariantName(self, parentVariantHelper, currentConfiguration, variantHelper):
        parentVariantList = parentVariantHelper.NormalVariants
        variantList = variantHelper.NormalVariants
        filteredVariantList = []
        parentVariantIndex = 0
        for entry in currentConfiguration:
            if self.__ContainsVariant(variantList, parentVariantList[parentVariantIndex].Name):
                filteredVariantList.append(entry)
            parentVariantIndex = parentVariantIndex + 1
        return "-".join(filteredVariantList)


    def __GenerateSLNPackageVariantConfig(self, variantHelper, snippet, package):
        if len(variantHelper.CartesianProduct) > 0:
            lines = []
            for entry in variantHelper.CartesianProduct:
                sectionVariantName = self.__BuildConfigurationName(entry)
                section = snippet.replace("##VARIANT##", sectionVariantName)
                lines.append(section)
            return "\n".join(lines)
        else:
            return snippet.replace("##VARIANT##", '')


    def __VariantSimpleReplace(self, variantHelper, snippet, package):
        if len(variantHelper.CartesianProduct) > 0:
            lines = []
            for entry in variantHelper.CartesianProduct:
                section = snippet
                sectionVariantName = self.__BuildConfigurationName(entry)
                section = section.replace("##VARIANT##", sectionVariantName)
                lines.append(section)
            return "\n".join(lines)
        else:
            return snippet.replace("##VARIANT##", '')

    def __GenerateProjectConfigurations(self, variantHelper, snippet, package):
        return self.__VariantSimpleReplace(variantHelper, snippet, package)


    def __GenerateImportGroups(self, variantHelper, snippet, package):
        return self.__VariantSimpleReplace(variantHelper, snippet, package)


    def __GenerateVariantPropertyGroups(self, variantHelper, snippet, package):
        if len(variantHelper.CartesianProduct) > 0:
            lines = []
            for entry in variantHelper.CartesianProduct:
                dynamicName = package.ResolvedVirtualVariantName
                section = self.__GetVariantPropertySections(snippet, dynamicName)
                sectionVariantName = self.__BuildConfigurationName(entry) 
                section = section.replace("##VARIANT##", sectionVariantName)
                lines.append(section)
            return "\n".join(lines)
        else:
            section = self.__GetVariantPropertySections(snippet, package.ResolvedVariantName)
            return section.replace("##VARIANT##", '')


    def __BuildGenerateCompilerSettingsGroupSection(self, snippet, package, includeDirs, cppDefines, variantExtDeps):
        includeDirs = self.__FixIncludes(includeDirs)
        includeDirs = self.__CreateVisualStudioStyleList(includeDirs)

        extLibDepVC = self.__GenerateVCExternalDependencies(package, variantExtDeps)

        cppLocalDefines = Util.ExtractNames(cppDefines)
        cppLocalDefineNames = self.__CreateVisualStudioStyleDefineList(cppLocalDefines)

        section = snippet
        section = section.replace("##PACKAGE_INCLUDE_DIRS##", includeDirs)
        section = section.replace("##PACKAGE_CPP_FLAGS##", cppLocalDefineNames)
        section = section.replace("##PACKAGE_EXTERNAL_LIBRARY_DEPENDENCIES##", extLibDepVC)
        return section


    def AddOption(self, option, includeDirs, variantExtDeps, cppDefines):
        # add external dependencies
        for entry in option.ExternalDependencies:
            if entry.Include != None and not entry.Include in includeDirs:
                includeDirs.append(entry.Include)
            if entry.IsFirstActualUse:
                variantExtDeps.append(entry)
        # add cpp defines
        for entry in option.DirectCPPDefines:
            cppDefines.append(entry)


    def __GenerateCompilerSettingsGroups(self, variantHelper, snippet, package):
        includeDirs = package.ResolvedBuildAllIncludeDirs
        cppDefines = package.ResolvedBuildAllCPPDefines

        # Process virtual variants
        variantExtDeps = self.__GetExternalLibraryDependencies(package)
        if variantHelper.VariantTypeDict.has_key(VariantType.Virtual):
            for variant in variantHelper.VariantTypeDict[VariantType.Virtual]:
                if not len(variant.Options) == 1: 
                    raise InternalErrorException("The virtual variant does not contain the expected option");
                self.AddOption(variant.Options[0], includeDirs, variantExtDeps, cppDefines)

        if len(variantHelper.CartesianProduct) > 0:
            orgIncludeDirs = includeDirs
            orgVariantExtDeps = variantExtDeps
            orgCppDefines = cppDefines

            lines = []
            for entry in variantHelper.CartesianProduct:
                includeDirs = list(orgIncludeDirs)
                variantExtDeps = list(orgVariantExtDeps)
                cppDefines = list(orgCppDefines)

                variantList = variantHelper.NormalVariants
                variantIdx = 0
                for optionName in entry:
                    variant = variantList[variantIdx]
                    option = variant.OptionDict[optionName]
                    variantIdx = variantIdx + 1
                    self.AddOption(option, includeDirs, variantExtDeps, cppDefines)

                dynamicName = package.ResolvedVirtualVariantName
                section = self.__BuildGenerateCompilerSettingsGroupSection(snippet, package, includeDirs, cppDefines, variantExtDeps)
                sectionVariantName = self.__BuildConfigurationName(entry)
                section = section.replace("##VARIANT##", sectionVariantName)
                lines.append(section)
            return "\n".join(lines)
        else:
            section = self.__BuildGenerateCompilerSettingsGroupSection(snippet, package, includeDirs, cppDefines, variantExtDeps)
            return section.replace("##VARIANT##", '')

    def __GetVariantPropertySections(self, snippet, variantName):
        str = snippet.replace("##PROJECT_VARIANT_NAME##", variantName)
        return str


    def __FixIncludes(self, entries):
        res = []
        for entry in entries:
            if entry == 'include':
                entry = "$(ProjectDir)\\include"
            res.append(entry)
        return res


    def __GenerateSLNDependencies1(self, package):
        str = ""
        if len(package.ResolvedDirectDependencies) > 0:
            str += '\tProjectSection(ProjectDependencies) = postProject\n'
            for dep in package.ResolvedDirectDependencies:
                if not dep.Package.IsVirtual and dep.Package.ResolvedPlatform != None:
                    str += '\t\t{%s} = {%s}\n' % (dep.Package.ResolvedPlatform.ProjectId, dep.Package.ResolvedPlatform.ProjectId)
            str += '\tEndProjectSection\n'
        return str


    def __GenerateSLNDependencies2(self, config, package):
        str = ""
        for entry in package.ResolvedBuildOrder:
            if not entry.IsVirtual and entry.ResolvedPlatform != None and package != entry:
                projectName = entry.Name
                projectPath = "%s\\%s.vcxproj" % (config.ToDosPath(entry.AbsolutePath), projectName)
                projectId = entry.ResolvedPlatform.ProjectId
                str += '\nProject("{8BC9CEB8-8B4A-11D0-8D11-00A0C91BC942}") = "%s", "%s", "{%s}"\n' % (projectName, projectPath, projectId)
                str += "EndProject"
        return str


    def __GenerateSLNPackageDependency(XmlUnsupportedPlatformException, snippet, package, sectionVariantName1, sectionVariantName2):
        section = snippet
        section = section.replace("##PACKAGE_PLATFORM_PROJECT_ID##",  package.ResolvedPlatform.ProjectId)
        section = section.replace("##VARIANT1##", sectionVariantName1)
        section = section.replace("##VARIANT2##", sectionVariantName2)
        return section


    def __GenerateSLNPackageVariants(self, variantHelper, snippet):
        if len(variantHelper.CartesianProduct) > 0:
            lines = []
            for entry in variantHelper.CartesianProduct:
                sectionVariantName = self.__BuildConfigurationName(entry)
                section = self.__GenerateSLNPackageDependency(snippet, variantHelper.Package, sectionVariantName, sectionVariantName)
                lines.append(section)
            return "\n".join(lines)
        else:
            return self.__GenerateSLNPackageDependency(snippet, variantHelper.Package, '', '')

    def __GenerateAndMatchSLNPackageVariants(self, parentVariantHelper, variantHelper, snippet):
        if len(parentVariantHelper.CartesianProduct) > 0:
            lines = []
            for entry in parentVariantHelper.CartesianProduct:
                sectionVariantName = self.__BuildConfigurationName(entry)
                matchedVariantName = self.__BuildMatchedVariantName(parentVariantHelper, entry, variantHelper)
                section = self.__GenerateSLNPackageDependency(snippet, variantHelper.Package, sectionVariantName, matchedVariantName)
                lines.append(section)
            return "\n".join(lines)
        else:
            return self.__GenerateSLNPackageDependency(snippet, variantHelper.Package, '', '')



    def __GenerateSLNDependencies3(self, variantHelper, snippet, package):
        res = []
        for entry in package.ResolvedBuildOrder:
            if not entry.IsVirtual and entry.ResolvedPlatform != None and package != entry:
                entryVariantHelper = VariantHelper(entry)
                section = self.__GenerateAndMatchSLNPackageVariants(variantHelper, entryVariantHelper, snippet)
                res.append(section)
        return "\n".join(res) + "\n" if len(res) > 0 else ""


    def __GenerateVCDependencies(self, snippet, config, package):
        res = []
        if len(package.ResolvedBuildOrder) > 1:
            for entry in package.ResolvedBuildOrder:
                if not entry.IsVirtual and entry.ResolvedPlatform != None and package != entry:
                    projectPath = "%s\\%s.vcxproj" % (config.ToPath(entry.AbsolutePath).replace('/', '\\'), entry.Name)
                    projectId = entry.ResolvedPlatform.ProjectId
                    str = snippet.replace("##PACKAGE_DEPENDENCY_PROJECT_PATH##",projectPath)
                    str = str.replace("##PACKAGE_DEPENDENCY_PROJECTID##", projectId.lower())
                    res.append(str)
        return "\n".join(res)


    def __GenerateVCExternalDependencies(self, package, variantExtDeps):
        str = ""
        extDeps = Util.FilterByType(package.ResolvedDirectExternalDependencies, ExternalDependencyType.StaticLib) 
        extDeps2 = Util.FilterByType(variantExtDeps, ExternalDependencyType.StaticLib) 
        extDeps += extDeps2
        if len(extDeps) > 0:
            additionalDependencies = set()
            additionalLibraryDirectories = set()
            for entry in extDeps:
                additionalDependencies.add(entry.Name)
                if entry.Location != None:
                    additionalLibraryDirectories.add(entry.Location)
            str += '\n    <Lib>\n'
            if len(additionalDependencies) > 0:
                additionalDependencies = self.__CreateVisualStudioStyleList(additionalDependencies)
                str += '      <AdditionalDependencies>%s</AdditionalDependencies>\n' % (additionalDependencies)
            if len(additionalLibraryDirectories) > 0:
                additionalLibraryDirectories = self.__CreateVisualStudioStyleList(additionalLibraryDirectories)
                str += '      <AdditionalLibraryDirectories>%s</AdditionalLibraryDirectories>\n' % (additionalLibraryDirectories)
            str += '    </Lib>'
        return str


    def __GetExternalLibraryDependencies(self, package):
        libPaths = []
        for entry in package.ResolvedBuildOrder:
            externalList = Util.FilterByType(entry.ResolvedDirectExternalDependencies, ExternalDependencyType.StaticLib)
            libPaths += externalList
        return libPaths
