#!/usr/bin/env python
# -*- coding: utf-8 -*-

#****************************************************************************************************************************************************
# Copyright (c) 2014 Freescale Semiconductor, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright notice,
#      this list of conditions and the following disclaimer.
#
#    * Redistributions in binary form must reproduce the above copyright notice,
#      this list of conditions and the following disclaimer in the documentation
#      and/or other materials provided with the distribution.
#
#    * Neither the name of the Freescale Semiconductor, Inc. nor the names of
#      its contributors may be used to endorse or promote products derived from
#      this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#****************************************************************************************************************************************************

import argparse
import ntpath
import os
import re
import shutil

g_templatePath = "Templates/Projects"

g_templatePathGLES2 = "GLES2"
g_templatePathGLES3 = "GLES3"
g_templatePathOpenVG = "OpenVG"
g_templatePathG2D = "G2D"

g_templatePathAndroid = "Android"
g_templateCopyDirAndroid = "Android"
g_templateRenameDirAndroid = os.path.join(g_templateCopyDirAndroid, "src/com/freescale/demo")
g_templateRenameDirAndroidNew = os.path.join(g_templateCopyDirAndroid, "src/com/freescale")
g_templateFileAndroidF1 = "Android.mk"
g_templateFileAndroidF2 = os.path.join(g_templateCopyDirAndroid, "jni/Android.mk")
g_templateFileAndroidF3 = os.path.join(g_templateCopyDirAndroid, "jni/DemoNativeActivity.cpp")
g_templateFileAndroidF4 = os.path.join(g_templateCopyDirAndroid, "AndroidManifest.xml")
g_templateFileAndroidF5 = os.path.join(g_templateCopyDirAndroid, "build.xml")
g_templateFileAndroidF6 = os.path.join(g_templateCopyDirAndroid, "res/values/strings.xml")
g_templateFileAndroidF7 = os.path.join(g_templateCopyDirAndroid, "src/com/freescale/demo/DemoApplication.java")
g_templateFileAndroidF8 = os.path.join(g_templateCopyDirAndroid, "src/com/freescale/demo/DemoNativeActivity.java")
g_templateFileAndroidF9 = os.path.join(g_templateCopyDirAndroid, "src/com/freescale/demo/NDKHelper.java")
g_replaceAndroid_PROJECT_NAME = "%AG_PROJECT_NAME%"
g_replaceAndroid_PROJECT_NAME_LOWERCASE = "%AG_PROJECT_NAME_L%"
g_replaceAndroid_PREFIXED_PROJECT_NAME = "%AG_PREFIXED_PROJECT_NAME%"
g_replaceAndroid_PREFIXED_PROJECT_NAME_LOWERCASE = "%AG_PREFIXED_PROJECT_NAME_L%"

g_templatePathVC2013 = "VC2013"
g_replaceVC2013_PROJECT_NAME = "%AG_PROJECT_NAME%"
g_replaceVC2013_PROJECT_ID = "%AG_VS_PROJECT_ID%"
g_defaultVCID = 'F73214FE-7A4B-4D7D-89EC-416B25E643BF'

g_templatePathCode = "Code"
g_replaceCODE_PROJECT_NAME = "%AG_PROJECT_NAME%"
g_replaceCODE_PROJECT_NAME_U = "%AG_PROJECT_NAME_U%"

g_templatePathFslGen = "FslGen"
g_replaceFslGen_PROJECT_NAME = "%AG_PROJECT_NAME%"
g_replaceFslGen_PROJECT_ID = "%AG_VS_PROJECT_ID%"

g_projectSource = "source"

g_reservedProjectNames = [
    "FslBase",
    "FslGraphics",
    "FslGraphicsEGL",
    "FslDemoPlatform",
    "FslDemoHostEGL",
    "FslGraphicsGLES2",
    "FslDemoApp",
    "FslDemoHost",
    "DemoNativeActivity",
    "ndk_helper",
    "native_app_glue",
    "cpufeatures",
]

def EnsureUTF8(str):
    try:
        return unicode(str.encode('utf-8'))
    except ValueError:
        return str


class ArgumentError(Exception):
  """Indicate that a invalid argument was supplied
  """

class FileConfig():
    def __init__(self, srcFile, dstFile):
        self.SrcFile = srcFile
        self.DstFile = dstFile

class ConfigAndroid():
    def __init__(self, templatePath, projectPath, projectName, prefixedProjectName):
        self.CopyDirectory = FileConfig(os.path.join(templatePath, g_templateCopyDirAndroid), os.path.join(projectPath, g_templateCopyDirAndroid))
        self.Files = [ FileConfig(os.path.join(templatePath, g_templateFileAndroidF1), os.path.join(projectPath, g_templateFileAndroidF1)),
                       FileConfig(os.path.join(projectPath, g_templateFileAndroidF2), os.path.join(projectPath, g_templateFileAndroidF2)),
                       FileConfig(os.path.join(projectPath, g_templateFileAndroidF3), os.path.join(projectPath, g_templateFileAndroidF3)),
                       FileConfig(os.path.join(projectPath, g_templateFileAndroidF4), os.path.join(projectPath, g_templateFileAndroidF4)),
                       FileConfig(os.path.join(projectPath, g_templateFileAndroidF5), os.path.join(projectPath, g_templateFileAndroidF5)),
                       FileConfig(os.path.join(projectPath, g_templateFileAndroidF6), os.path.join(projectPath, g_templateFileAndroidF6)),
                       FileConfig(os.path.join(projectPath, g_templateFileAndroidF7), os.path.join(projectPath, g_templateFileAndroidF7)),
                       FileConfig(os.path.join(projectPath, g_templateFileAndroidF8), os.path.join(projectPath, g_templateFileAndroidF8)),
                       FileConfig(os.path.join(projectPath, g_templateFileAndroidF9), os.path.join(projectPath, g_templateFileAndroidF9)) ]
        newDirName = os.path.join(g_templateRenameDirAndroidNew, prefixedProjectName.lower())
        self.RenameDirectory = FileConfig(os.path.join(projectPath, g_templateRenameDirAndroid), os.path.join(projectPath, newDirName))

class ConfigVC2013():
    def __init__(self, templatePath, projectPath, projectName, prefixedProjectName, projectPrefix):
        templateFileVC2013GLES2_F1 = "TemplateVC2013.sln"
        templateFileVC2013GLES2_F2 = "TemplateVC2013.vcxproj"
        f1 = '%s.sln' % (projectName)
        f2 = '%s.vcxproj' % (projectName)
        if projectPrefix.lower().startswith("openvg_"):
            templateFileVC2013GLES2_B1 = ".StartProject_Mesa.bat"
            self.Files = [ FileConfig(os.path.join(templatePath, templateFileVC2013GLES2_F1), os.path.join(projectPath, f1)),
                           FileConfig(os.path.join(templatePath, templateFileVC2013GLES2_F2), os.path.join(projectPath, f2)),
                           FileConfig(os.path.join(templatePath, templateFileVC2013GLES2_B1), os.path.join(projectPath, templateFileVC2013GLES2_B1)),
                         ]
        else:
            templateFileVC2013GLES2_B1 = ".StartProject_Arm.bat"
            templateFileVC2013GLES2_B2 = ".StartProject_Vivante.bat"
            #templateFileVC2013GLES2_B3 = ".StartProject_Mesa.bat"
            self.Files = [ FileConfig(os.path.join(templatePath, templateFileVC2013GLES2_F1), os.path.join(projectPath, f1)),
                           FileConfig(os.path.join(templatePath, templateFileVC2013GLES2_F2), os.path.join(projectPath, f2)),
                           FileConfig(os.path.join(templatePath, templateFileVC2013GLES2_B1), os.path.join(projectPath, templateFileVC2013GLES2_B1)),
                           FileConfig(os.path.join(templatePath, templateFileVC2013GLES2_B2), os.path.join(projectPath, templateFileVC2013GLES2_B2)),
                           #FileConfig(os.path.join(templatePath, templateFileVC2013GLES2_B3), os.path.join(projectPath, templateFileVC2013GLES2_B3)),
                         ]

class ConfigCode():
    def __init__(self, templatePath, projectPath, projectName, prefixedProjectName):
        f1 = 'source/%s_Register.cpp' % (projectName)
        f2 = 'source/%s.hpp' % (projectName)
        f3 = 'source/%s.cpp' % (projectName)
        gitIgnore = '.gitignore'
        self.Files = [ FileConfig(os.path.join(templatePath, 'source/DemoAppTemplate_Register.cpp'), os.path.join(projectPath, f1)),
                       FileConfig(os.path.join(templatePath, 'source/DemoAppTemplate.hpp'), os.path.join(projectPath, f2)),
                       FileConfig(os.path.join(templatePath, 'source/DemoAppTemplate.cpp'), os.path.join(projectPath, f3)),
                       FileConfig(os.path.join(templatePath, gitIgnore), os.path.join(projectPath, gitIgnore)),
                     ]

class ConfigFslGen():
    def __init__(self, templatePath, projectPath, projectName, prefixedProjectName):
        f1 = 'Fsl.gen'
        gitIgnore = '.gitignore'
        self.Files = [ FileConfig(os.path.join(templatePath, 'Fsl.gen'), os.path.join(projectPath, f1)),
                     ]

class ConfigVariant():
    def __init__(self, templatePath, projectPath, projectName, projectPrefix):
        self.ProjectName = projectName
        self.ProjectPath = projectPath
        self.PrefixedProjectName = projectPrefix + projectName
        self.ProjectPrefix = projectPrefix
        self.ProjectPathSource = os.path.join(projectPath, g_projectSource)
        self.TemplatePathAndroid = os.path.join(templatePath, g_templatePathAndroid)
        self.TemplatePathCode = os.path.join(templatePath, g_templatePathCode)
        self.TemplatePathVC2013 = os.path.join(templatePath, g_templatePathVC2013)
        self.TemplatePathFslGen = os.path.join(templatePath, g_templatePathFslGen)
        self.ConfigCode = ConfigCode(self.TemplatePathCode, projectPath, projectName, self.PrefixedProjectName)
        self.ConfigAndroid = ConfigAndroid(self.TemplatePathAndroid, projectPath, projectName, self.PrefixedProjectName)
        self.ConfigVC2013 = ConfigVC2013(self.TemplatePathVC2013, projectPath, projectName, self.PrefixedProjectName, projectPrefix)
        self.ConfigFslGen = ConfigFslGen(self.TemplatePathFslGen, projectPath, projectName, self.PrefixedProjectName)

class Config():
    def __init__(self, fslGraphicsSDKPath, projectPath, projectName, projectType):
        self.ValidateProjectName(projectName)
        self.FslGraphicsSDKPath = fslGraphicsSDKPath
        self.ProjectName = projectName;
        self.ProjectPath = os.path.join(projectPath, projectName)
        self.TemplatePath = os.path.join(fslGraphicsSDKPath, g_templatePath)
        self.TemplatePathGLES2 = os.path.join(self.TemplatePath, g_templatePathGLES2)
        self.TemplatePathGLES3 = os.path.join(self.TemplatePath, g_templatePathGLES3)
        self.TemplatePathOpenVG = os.path.join(self.TemplatePath, g_templatePathOpenVG)
        self.TemplatePathG2D = os.path.join(self.TemplatePath, g_templatePathG2D)
        # Call this as the last thing since it references member variables
        self.ConfigVariant = self.PrepareProjectType(projectType)


    def ValidateProjectName(self, projectName):
        if len(projectName) < 1:
            raise ArgumentError("A project name needs to contain atleast one character");
        if re.match("[a-zA-Z0-9_]", projectName) == None:
            raise ArgumentError("A project name can only contain alpha numeric characters, digits and underscores '%s'" % projectName);
        if not projectName[0].isalpha():
            raise ArgumentError("A project name needs to start with a alpha character '%s'" % projectName);
        for name in g_reservedProjectNames:
            if projectName == name:
                raise ArgumentError("The given project name '%s' is reserved" % projectName);

    def PrepareProjectType(self, projectType):
        projectTypeLower = projectType.lower();
        if projectTypeLower == "gles2":
            return ConfigVariant(self.TemplatePathGLES2, self.ProjectPath, self.ProjectName, "GLES2_")
        elif projectTypeLower == "gles3":
            return ConfigVariant(self.TemplatePathGLES3, self.ProjectPath, self.ProjectName, "GLES3_")
        elif projectTypeLower == "openvg":
            return ConfigVariant(self.TemplatePathOpenVG, self.ProjectPath, self.ProjectName, "OPENVG_")
        elif projectTypeLower == "g2d":
            return ConfigVariant(self.TemplatePathG2D, self.ProjectPath, self.ProjectName, "G2D_")
        else:
             raise ArgumentError("Unknown project type '%s'. Expected: GLES2, GLES3, OpenVG or G2D." % projectType);


def ReadFile(filename):
    content = None
    with open (filename, "r") as theFile:
        content=theFile.read()
    return content


def SaveFile(filename, content):
    with open (filename, "w") as theFile:
        theFile.write(content)


def ProcessCodeTemplate(filename, projectName, prefixedProjectName):
    content = ReadFile(filename)
    content = content.replace(g_replaceCODE_PROJECT_NAME, projectName)
    return content.replace(g_replaceCODE_PROJECT_NAME_U, projectName.upper())


def ProcessAndroidTemplate(filename, projectName, prefixedProjectName):
    content = ReadFile(filename)
    content = content.replace(g_replaceAndroid_PREFIXED_PROJECT_NAME_LOWERCASE, prefixedProjectName.lower())
    content = content.replace(g_replaceAndroid_PREFIXED_PROJECT_NAME, prefixedProjectName)
    content = content.replace(g_replaceAndroid_PROJECT_NAME_LOWERCASE, projectName.lower())
    return content.replace(g_replaceAndroid_PROJECT_NAME, projectName)


def ProcessVC2013Template(filename, projectName, prefixedProjectName, projectId):
    content = ReadFile(filename)
    content = content.replace(g_replaceVC2013_PROJECT_NAME, projectName)
    return content.replace(g_replaceVC2013_PROJECT_ID, projectId)


def ProcessFslGenTemplate(filename, projectName, prefixedProjectName, projectId):
    content = ReadFile(filename)
    content = content.replace(g_replaceFslGen_PROJECT_NAME, projectName)
    content = content.replace(g_replaceFslGen_PROJECT_ID, projectId)    
    return content


def CmdCode(configVariant):
    print('Preparing project template code')
    for fileConfig in configVariant.ConfigCode.Files:
        content = ProcessCodeTemplate(fileConfig.SrcFile, configVariant.ProjectName, configVariant.PrefixedProjectName)
        SaveFile(fileConfig.DstFile, content)


def CmdAndroid(configVariant):
    print('Preparing Android project')
    shutil.copytree(configVariant.ConfigAndroid.CopyDirectory.SrcFile, configVariant.ConfigAndroid.CopyDirectory.DstFile)
    for fileConfig in configVariant.ConfigAndroid.Files:
        content = ProcessAndroidTemplate(fileConfig.SrcFile, configVariant.ProjectName, configVariant.PrefixedProjectName)
        SaveFile(fileConfig.DstFile, content)
    os.rename(configVariant.ConfigAndroid.RenameDirectory.SrcFile, configVariant.ConfigAndroid.RenameDirectory.DstFile)


def CmdVC2013(configVariant, projectId):
    print('Preparing VC2013 project')
    for fileConfig in configVariant.ConfigVC2013.Files:
        content = ProcessVC2013Template(fileConfig.SrcFile, configVariant.ProjectName, configVariant.PrefixedProjectName, projectId)
        SaveFile(fileConfig.DstFile, content)

def CmdFslGen(configVariant, projectId):
    print('Preparing FslGen project')
    for fileConfig in configVariant.ConfigFslGen.Files:
        content = ProcessFslGenTemplate(fileConfig.SrcFile, configVariant.ProjectName, configVariant.PrefixedProjectName, projectId)
        SaveFile(fileConfig.DstFile, content)

def CmdAll(configVariant, projectId):
    CmdAndroid(configVariant)
    CmdVC2013(configVariant, projectId)
    CmdFslGen(configVariant, projectId)

def AddArgSubParser(subparsers, cmdName, cmdHelp):
    parser = subparsers.add_parser(cmdName, help=cmdHelp)
    parser.add_argument("projectName", help="The name of the new project")
    parser.add_argument('-t', '--type', required=True,  help='Select demo type: GLES2, GLES3, OpenVG, G2D')
    parser.set_defaults(which=cmdName)
    return parser

def Main():
    currentDir = os.getcwd()
    currentDir = EnsureUTF8(currentDir)

    cmdAll = 'all'
    cmdVC2013 = 'vc2013'
    cmdAndroid = 'android'
    cmdFslGen = 'fslgen'

    parser = argparse.ArgumentParser(description='Create the project files for a DemoApp for the various supported platforms.')
    subparsers = parser.add_subparsers(title='subcommands',  description='valid subcommands', help='additional help')
    allParser = AddArgSubParser(subparsers, cmdAll, 'Create all build configurations')
    allParser.add_argument('-i', default=g_defaultVCID, help='Visual studio project id')

    vc2013Parser = AddArgSubParser(subparsers, cmdVC2013, 'Create project files for VC2013')
    vc2013Parser.add_argument('-i', default=g_defaultVCID,  help='Visual studio project id')

    androidParser = AddArgSubParser(subparsers, cmdAndroid, 'Create project files for Android')

    fslGenParser = AddArgSubParser(subparsers, cmdFslGen, 'Create project files for FslGen')

    print('FslNewDemoProject V0.2.4')
    try:
        fslGraphicsSDKPath = os.environ.get('FSL_GRAPHICS_SDK')
        if fslGraphicsSDKPath == None:
            raise EnvironmentError("FSL_GRAPHICS_SDK environment variable not set");
        fslGraphicsSDKPath = EnsureUTF8(fslGraphicsSDKPath)

        args = parser.parse_args()

        config = Config(fslGraphicsSDKPath, currentDir, args.projectName, args.type)
        configVariant = config.ConfigVariant;

        if os.path.isdir(configVariant.ProjectPath):
            raise EnvironmentError("The project directory already exist: '%s'" % (configVariant.ProjectPath));
        elif os.path.exists(configVariant.ProjectPath):
            raise EnvironmentError("A file named '%s' already exist." % (configVariant.ProjectPath));
        else:
            os.mkdir(configVariant.ProjectPath)
            os.mkdir(configVariant.ProjectPathSource)

        if args.which == cmdAll:
            CmdCode(configVariant)
            CmdAll(configVariant, args.i)
        if args.which == cmdVC2013:
            CmdCode(configVariant)
            CmdVC2013(configVariant, args.i)
        if args.which == cmdAndroid:
            CmdCode(configVariant)
            CmdAndroid(configVariant)
        if args.which == cmdFslGen:
            CmdCode(configVariant)
            CmdFslGen(configVariant, args.i)

    except (EnvironmentError) as ex:
        print "ERROR: %s" % ex.message
        raise ex

Main()
