/****************************************************************************************************************************************************
 * Copyright (c) 2015 Freescale Semiconductor, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *
 *    * Redistributions in binary form must reproduce the above copyright notice,
 *      this list of conditions and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 *    * Neither the name of the Freescale Semiconductor, Inc. nor the names of
 *      its contributors may be used to endorse or promote products derived from
 *      this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ****************************************************************************************************************************************************/

#include "FontTest.hpp"
#include <cassert>

namespace Fsl
{
  namespace
  {
    FontGlyphRange g_ranges[] =
    {
      FontGlyphRange(32, 96),
    };


    FontGlyphBasicKerning g_glyphKerning[] =
    {
      FontGlyphBasicKerning(0, 0, 14),
      FontGlyphBasicKerning(-2, 7, 17),
      FontGlyphBasicKerning(-1, 3, 17),
      FontGlyphBasicKerning(-3, 7, 21),
      FontGlyphBasicKerning(-2, 4, 21),
      FontGlyphBasicKerning(-1, 6, 36),
      FontGlyphBasicKerning(-2, 7, 32),
      FontGlyphBasicKerning(-3, 3, 10),
      FontGlyphBasicKerning(-3, 2, 12),
      FontGlyphBasicKerning(-3, 2, 12),
      FontGlyphBasicKerning(-1, 2, 21),
      FontGlyphBasicKerning(1, 9, 30),
      FontGlyphBasicKerning(-1, 27, 11),
      FontGlyphBasicKerning(-3, 18, 14),
      FontGlyphBasicKerning(-1, 28, 11),
      FontGlyphBasicKerning(-3, 5, 18),
      FontGlyphBasicKerning(-2, 7, 21),
      FontGlyphBasicKerning(0, 7, 21),
      FontGlyphBasicKerning(-2, 7, 21),
      FontGlyphBasicKerning(-2, 7, 21),
      FontGlyphBasicKerning(-2, 7, 21),
      FontGlyphBasicKerning(-2, 6, 21),
      FontGlyphBasicKerning(-2, 7, 21),
      FontGlyphBasicKerning(-1, 6, 21),
      FontGlyphBasicKerning(-2, 7, 21),
      FontGlyphBasicKerning(-2, 7, 21),
      FontGlyphBasicKerning(-1, 15, 14),
      FontGlyphBasicKerning(-1, 14, 14),
      FontGlyphBasicKerning(0, 11, 28),
      FontGlyphBasicKerning(1, 18, 30),
      FontGlyphBasicKerning(0, 11, 28),
      FontGlyphBasicKerning(-2, 6, 22),
      FontGlyphBasicKerning(-1, -2, 43),
      FontGlyphBasicKerning(-3, 0, 36),
      FontGlyphBasicKerning(0, 0, 26),
      FontGlyphBasicKerning(-1, 0, 31),
      FontGlyphBasicKerning(0, 0, 32),
      FontGlyphBasicKerning(0, 0, 25),
      FontGlyphBasicKerning(0, 0, 23),
      FontGlyphBasicKerning(-2, 0, 35),
      FontGlyphBasicKerning(0, 0, 34),
      FontGlyphBasicKerning(0, 0, 14),
      FontGlyphBasicKerning(-3, 0, 18),
      FontGlyphBasicKerning(0, 0, 31),
      FontGlyphBasicKerning(0, 0, 24),
      FontGlyphBasicKerning(-2, 0, 42),
      FontGlyphBasicKerning(-2, 0, 37),
      FontGlyphBasicKerning(-1, 0, 39),
      FontGlyphBasicKerning(-2, 0, 23),
      FontGlyphBasicKerning(-1, 0, 39),
      FontGlyphBasicKerning(0, 0, 28),
      FontGlyphBasicKerning(-1, 0, 23),
      FontGlyphBasicKerning(-2, 0, 28),
      FontGlyphBasicKerning(0, 0, 36),
      FontGlyphBasicKerning(-3, 0, 35),
      FontGlyphBasicKerning(-3, -1, 43),
      FontGlyphBasicKerning(-3, 0, 31),
      FontGlyphBasicKerning(-3, 0, 29),
      FontGlyphBasicKerning(-2, 0, 28),
      FontGlyphBasicKerning(-1, 3, 12),
      FontGlyphBasicKerning(-3, 5, 14),
      FontGlyphBasicKerning(-1, 3, 12),
      FontGlyphBasicKerning(-3, 6, 19),
      FontGlyphBasicKerning(-3, 37, 25),
      FontGlyphBasicKerning(6, 1, 28),
      FontGlyphBasicKerning(-2, 10, 21),
      FontGlyphBasicKerning(-1, 0, 24),
      FontGlyphBasicKerning(-2, 10, 21),
      FontGlyphBasicKerning(-1, 0, 25),
      FontGlyphBasicKerning(-2, 10, 22),
      FontGlyphBasicKerning(-1, 0, 14),
      FontGlyphBasicKerning(-2, 10, 22),
      FontGlyphBasicKerning(-1, 0, 24),
      FontGlyphBasicKerning(-1, 1, 11),
      FontGlyphBasicKerning(-4, 1, 11),
      FontGlyphBasicKerning(-1, 0, 22),
      FontGlyphBasicKerning(-1, 0, 11),
      FontGlyphBasicKerning(-1, 10, 38),
      FontGlyphBasicKerning(-1, 10, 24),
      FontGlyphBasicKerning(-2, 10, 25),
      FontGlyphBasicKerning(-1, 10, 25),
      FontGlyphBasicKerning(-2, 10, 24),
      FontGlyphBasicKerning(-1, 10, 17),
      FontGlyphBasicKerning(-2, 10, 16),
      FontGlyphBasicKerning(-3, 5, 15),
      FontGlyphBasicKerning(-2, 10, 24),
      FontGlyphBasicKerning(-3, 10, 22),
      FontGlyphBasicKerning(-3, 10, 32),
      FontGlyphBasicKerning(-4, 10, 22),
      FontGlyphBasicKerning(-3, 10, 23),
      FontGlyphBasicKerning(-3, 10, 17),
      FontGlyphBasicKerning(2, 0, 24),
      FontGlyphBasicKerning(-2, 5, 6),
      FontGlyphBasicKerning(4, 0, 24),
      FontGlyphBasicKerning(-3, 17, 27),
      FontGlyphBasicKerning(0, 0, 0),
    };
  }


  UTF8String FontTest::GetName() const
  {
    return UTF8String("FontTest");
  }


  UTF8String FontTest::GetPathName() const
  {
    return UTF8String("FontChars_Font");
  }


  FontDesc FontTest::GetDesc() const
  {
    return FontDesc(50, 37, Point2(1, 0));
  }


  int32_t FontTest::RangeCount() const
  {
    return sizeof(g_ranges)/sizeof(FontGlyphRange);
  }


  FontGlyphRange FontTest::GetRange(const int32_t index) const
  {
    assert(index >= 0 && index <= RangeCount());
    return g_ranges[index];
  }


  int32_t FontTest::Count() const
  {
    return sizeof(g_glyphKerning)/sizeof(FontGlyphBasicKerning);
  }


  FontGlyphBasicKerning FontTest::Get(const int32_t index) const
  {
    assert(index >= 0 && index <= Count());
    return g_glyphKerning[index];
  }
}
