#ifndef FSLDEMOAPP_SERVICE_IMAGE_IIMAGESERVICE_HPP
#define FSLDEMOAPP_SERVICE_IMAGE_IIMAGESERVICE_HPP
/****************************************************************************************************************************************************
* Copyright (c) 2014 Freescale Semiconductor, Inc.
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*
*    * Redistributions of source code must retain the above copyright notice,
*      this list of conditions and the following disclaimer.
*
*    * Redistributions in binary form must reproduce the above copyright notice,
*      this list of conditions and the following disclaimer in the documentation
*      and/or other materials provided with the distribution.
*
*    * Neither the name of the Freescale Semiconductor, Inc. nor the names of
*      its contributors may be used to endorse or promote products derived from
*      this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
* ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
****************************************************************************************************************************************************/

#include <FslBase/BasicTypes.hpp>
#include <FslGraphics/PixelFormat.hpp>
#include <FslBase/IO/Path.hpp>

namespace Fsl
{
  class Bitmap;

  //! @brief The image service is a optional service.
  //! @note While its optional its always present on platforms that support the full demo framework.
  class IImageService
  {
  public:
    virtual ~IImageService() {}

    //! @brief Try to read the supplied file
    //! @param path the path to load the file from
    //! @return true on success, false if the image failed to load (for any reason)
    virtual bool TryRead(Bitmap& rBitmap, const IO::Path& path) = 0;

    //! @brief Try to read the content of the file as a bitmap.
    //! @param path the path to load the file from
    //! @param desiredPixelFormat the pixel format that the bitmap should be using 
    //         (if the source image uses a different pixel format it will be converted to the desiredPixelFormat if possible).
    //! @return true on success, false if the image failed to load (for any reason)
    virtual bool TryRead(Bitmap& rBitmap, const IO::Path& path, const PixelFormat::Enum desiredPixelFormat) = 0;

    // TODO: we probably need functions for loading exotic formats as well (formats that can't be contained in Bitmap since it might be compressed)
    
    
    // TODO: we might want to add a bitmap conversion methods here
    //! @brief Convert the bitmap to the desired pixel format else return false.
    //virtual bool TryConvert(Bitmap& rBitmap, const PixelFormat::Enum desiredPixelFormat) = 0;

    //! @brief Convert the bitmap to the desired pixel format (if possible, else throw a exception)
    //virtual void Convert(Bitmap& rBitmap, const PixelFormat::Enum desiredPixelFormat) = 0;
  };
}

#endif
