/****************************************************************************************************************************************************
* Copyright (c) 2014 Freescale Semiconductor, Inc.
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*
*    * Redistributions of source code must retain the above copyright notice,
*      this list of conditions and the following disclaimer.
*
*    * Redistributions in binary form must reproduce the above copyright notice,
*      this list of conditions and the following disclaimer in the documentation
*      and/or other materials provided with the distribution.
*
*    * Neither the name of the Freescale Semiconductor, Inc. nor the names of
*      its contributors may be used to endorse or promote products derived from
*      this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
* ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
****************************************************************************************************************************************************/

#include <iostream>
#include <FslBase/Getopt/OptionBaseValues.hpp>
#include <FslBase/String/StringParseUtil.hpp>
#include <FslDemoPlatform/DemoHostManagerOptionParser.hpp>

namespace Fsl
{
  namespace
  {
    struct CommandId
    {
      enum Enum
      {
        ExitAfterFrame = DEMO_HOSTMANAGER_OPTION_BASE_INTERNAL,
        ScreenshotFrequency,
        LogStats,
        Stats,
        MMDC,
        EnableBasic2DPrealloc
      };
    };
  }

  DemoHostManagerOptionParser::DemoHostManagerOptionParser()
    : m_exitAfterFrame(-1)
    , m_screenshotFrequency(0)
    , m_logStats(false)
    , m_stats(false)
    , m_enableBasic2DPrealloc(true)
    , m_enableMMDC(false)
  {
  }


  void DemoHostManagerOptionParser::ArgumentSetup(std::deque<Option>& rOptions)
  {
    rOptions.push_back(Option("ExitAfterFrame", OptionArgument::OptionRequired, CommandId::ExitAfterFrame, "Exit after the given number of frames has been rendered"));
    rOptions.push_back(Option("ScreenshotFrequency", OptionArgument::OptionRequired, CommandId::ScreenshotFrequency, "Create a screenshot at the given frame frequency"));
    rOptions.push_back(Option("LogStats", OptionArgument::OptionNone, CommandId::LogStats, "Log basic rendering stats"));
    rOptions.push_back(Option("Stats", OptionArgument::OptionNone, CommandId::Stats, "Display basic frame profiling stats"));
    rOptions.push_back(Option("EnableBasic2DPrealloc", OptionArgument::OptionRequired, CommandId::EnableBasic2DPrealloc, "Enable/disable basic2d preallocation (Stats is enabled this is forced true)", OptionGroup::Hidden));

#ifdef FSL_PLATFORM_YOCTO
    rOptions.push_back(Option("mmdc", OptionArgument::OptionNone, CommandId::MMDC, "Enable basic MMDC profiling (experimental)"));
#endif
  }


  OptionParseResult::Enum DemoHostManagerOptionParser::Parse(const int cmdId, const char*const pszOptArg)
  {
    Rectangle rectValue;
    bool boolValue;
    switch (cmdId)
    {
    case CommandId::ExitAfterFrame:
      StringParseUtil::Parse(m_exitAfterFrame, pszOptArg);
      return OptionParseResult::Parsed;
    case CommandId::ScreenshotFrequency:
      StringParseUtil::Parse(m_screenshotFrequency, pszOptArg);
      return OptionParseResult::Parsed;
    case CommandId::LogStats:
      m_logStats = true;
      return OptionParseResult::Parsed;
    case CommandId::Stats:
      m_stats = true;
      return OptionParseResult::Parsed;
    case CommandId::MMDC:
      m_enableMMDC = true;
      return OptionParseResult::Parsed;
    case CommandId::EnableBasic2DPrealloc:
      StringParseUtil::Parse(boolValue, pszOptArg);
      m_enableBasic2DPrealloc = boolValue;
      return OptionParseResult::Parsed;
    default:
      break;
    }
    return OptionParseResult::NotHandled;
  }


  bool DemoHostManagerOptionParser::ParsingComplete()
  {
    return true;
  }


  int32_t DemoHostManagerOptionParser::GetExitAfterFrame() const
  {
    return m_exitAfterFrame;
  }


  uint32_t DemoHostManagerOptionParser::GetScreenshotFrequency() const
  {
    return m_screenshotFrequency;
  }


  bool DemoHostManagerOptionParser::IsLogStatsEnabled() const
  {
    return m_logStats;
  }


  bool DemoHostManagerOptionParser::IsStatsEnabled() const
  {
    return m_stats;
  }


  bool DemoHostManagerOptionParser::IsBasic2DPreallocEnabled() const
  {
    return m_stats || m_enableBasic2DPrealloc;
  }


  bool DemoHostManagerOptionParser::IsMMDCEnabled() const
  {
    return m_enableMMDC;
  }
}


