/****************************************************************************************************************************************************
* Copyright (c) 2014 Freescale Semiconductor, Inc.
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*
*    * Redistributions of source code must retain the above copyright notice,
*      this list of conditions and the following disclaimer.
*
*    * Redistributions in binary form must reproduce the above copyright notice,
*      this list of conditions and the following disclaimer in the documentation
*      and/or other materials provided with the distribution.
*
*    * Neither the name of the Freescale Semiconductor, Inc. nor the names of
*      its contributors may be used to endorse or promote products derived from
*      this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
* ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
****************************************************************************************************************************************************/

#include <FslDemoPlatform/Setup/DemoSetupManager.hpp>
#include <FslDemoApp/Setup/HostDemoAppSetup.hpp>
#include <FslDemoPlatform/Setup/DemoHostAppSetupBuilder.hpp>
#include <FslDemoPlatform/Setup/DemoHostRegistry.hpp>
#include <FslDemoHost/Service/ServicePriorityList.hpp>
#include <FslDemoHost/Service/ServiceRegistryImpl.hpp>
#include <FslDemoHost/Service/ServiceRegistry.hpp>
#include <FslDemoHost/Service/ThreadLocal/ThreadLocalServiceDeque.hpp>
#include <FslDemoHost/Service/ThreadLocal/ThreadLocalServiceHost.hpp>
#include <FslDemoHost/Service/ThreadLocal/ThreadLocalSingletonServiceFactoryTemplate.hpp>
#include <FslDemoHost/Service/Content/ContentManagerServiceFactory.hpp>
#include <FslDemoHost/Service/DemoAppControl/DemoAppControlServiceFactory.hpp>
#include <FslDemoHost/Service/Events/EventsService.hpp>
#include <FslDemoHost/Service/Graphics/GraphicsService.hpp>
#include <FslDemoHost/Service/Keyboard/KeyboardService.hpp>
#include <FslDemoHost/Service/Mouse/MouseService.hpp>
#include <FslDemoHost/Service/NativeWindowEvents/NativeWindowEventsService.hpp>
#include <FslDemoHost/Service/Profiler/ProfilerService.hpp>
#include <FslDemoHost/Service/Persistent/PersistentDataManagerServiceFactory.hpp>
#include <FslDemoHost/Service/Test/TestService.hpp>
#include <FslDemoPlatform/Service/DemoPlatformControl/DemoPlatformControl.hpp>
#include "../Configuration/PlatformConfig.hpp"
#include <FslDemoApp/Setup/RegisterDemoApp_fwd.hpp>
//#include <FslDemoPlatform/Service/MMDCStats/MMDCStatsService.hpp>

namespace Fsl
{
  
  //typedef ThreadLocalSingletonServiceFactoryTemplate<MMDCStatsService, IMMDCStatsService> MMDCStatsServiceFactory;

  typedef ThreadLocalSingletonServiceFactoryTemplate<DemoPlatformControl, IDemoPlatformControl> PlatformControlFactory;
  typedef ThreadLocalSingletonServiceFactoryTemplate2<EventsService, IEventService, IEventPoster> EventsFactory;
  typedef ThreadLocalSingletonServiceFactoryTemplate2<GraphicsService, IGraphicsService, IGraphicsServiceControl> GraphicsServiceFactory;
  typedef ThreadLocalSingletonServiceFactoryTemplate<KeyboardService, IKeyboard> KeyboardFactory;
  typedef ThreadLocalSingletonServiceFactoryTemplate<MouseService, IMouse> MouseFactory;
  typedef ThreadLocalSingletonServiceFactoryTemplate2<NativeWindowEventsService, INativeWindowEvents, INativeWindowEventSender> NativeWindowEventsFactory;
  typedef ThreadLocalSingletonServiceFactoryTemplate<TestService, ITestService> TestServiceFactory;
  typedef ThreadLocalSingletonServiceFactoryTemplate2<ProfilerService, IProfilerService, IProfilerServiceControl> ProfilerServiceFactory;

  DemoSetup DemoSetupManager::GetSetup(const DemoSetupManagerConfig& config, const bool verbose)
  {
    std::shared_ptr<ServiceRegistryImpl> theServiceRegistry(new ServiceRegistryImpl());
    ServiceRegistry serviceRegistry(theServiceRegistry);

    const std::shared_ptr<IThreadLocalSingletonServiceFactory> contentManagerServiceFactory(new ContentManagerServiceFactory(config.ContentPath));
    const std::shared_ptr<IThreadLocalSingletonServiceFactory> persistentDataManagerServiceFactory(new PersistentDataManagerServiceFactory(config.PersistentDataPath));
    const std::shared_ptr<IThreadLocalSingletonServiceFactory> demoAppControlServiceFactory(new DemoAppControlServiceFactory(EXIT_SUCCESS));

    serviceRegistry.Register(demoAppControlServiceFactory);
    serviceRegistry.Register(contentManagerServiceFactory);
    serviceRegistry.Register(persistentDataManagerServiceFactory);

    serviceRegistry.Register<GraphicsServiceFactory>();
    serviceRegistry.Register<KeyboardFactory>();
    serviceRegistry.Register<MouseFactory>();
    serviceRegistry.Register<EventsFactory>(ServicePriorityList::EventsService());
    serviceRegistry.Register<NativeWindowEventsFactory>(ServicePriorityList::NativeWindowEventsService());
    // Launch the test service with minimum priority ensuring it has access to all other services (except other min priority services)
    serviceRegistry.Register<TestServiceFactory>(ServicePriorityList::TestService());
    serviceRegistry.Register<PlatformControlFactory>(ServicePriorityList::PlatformControlService());
    serviceRegistry.Register<ProfilerServiceFactory>(ServicePriorityList::ProfilerService());

    //serviceRegistry.Register<MMDCStatsServiceFactory>(ServicePriorityList::MMDCStatsService());

    // Prepare the hosts
    DemoHostRegistry hostRegistry;
    PlatformConfig::Configure(hostRegistry, serviceRegistry);

    // Prepare the app and its desired host configuration.
    DemoHostAppSetupBuilder hostAppSetupBuilder;
    HostDemoAppSetup hostDemoAppSetup(hostRegistry, serviceRegistry, hostAppSetupBuilder);
    HOST_DemoAppSetup(&hostDemoAppSetup);

    const DemoHostAppSetup appSetup = hostAppSetupBuilder.GetSetup();
    const DemoHostSetup hostSetup = hostRegistry.GetSetup(*(appSetup.DemoHostFeatures));

    // Get the services ready
    ThreadLocalServiceDeque services;
    theServiceRegistry->LockAndExtractServices(services);
    theServiceRegistry.reset();

    ThreadLocalServiceHost serviceHost(services);

    return DemoSetup(serviceHost.GetProvider(), hostSetup, appSetup, verbose);
  }
}
