#ifndef FSLGRAPHICS_RAWBITMAPEX_HPP
#define FSLGRAPHICS_RAWBITMAPEX_HPP
/****************************************************************************************************************************************************
* Copyright (c) 2014 Freescale Semiconductor, Inc.
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*
*    * Redistributions of source code must retain the above copyright notice,
*      this list of conditions and the following disclaimer.
*
*    * Redistributions in binary form must reproduce the above copyright notice,
*      this list of conditions and the following disclaimer in the documentation
*      and/or other materials provided with the distribution.
*
*    * Neither the name of the Freescale Semiconductor, Inc. nor the names of
*      its contributors may be used to endorse or promote products derived from
*      this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
* ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
****************************************************************************************************************************************************/

#include <FslGraphics/RawBitmap.hpp>

namespace Fsl
{
  class RawBitmapEx
  {
    //! The pointer to the raw image data
    void* m_pContent;
    int32_t m_width;
    int32_t m_height;
    int32_t m_stride;
    PixelFormat::Enum m_pixelFormat;
  public:
    RawBitmapEx();
    RawBitmapEx(void*const pContent, const int32_t width, const int32_t height, const PixelFormat::Enum pixelFormat);
    RawBitmapEx(void*const pContent, const int32_t width, const int32_t height, const PixelFormat::Enum pixelFormat, const int32_t stride);

    bool IsValid() const { return m_pContent != nullptr; }

    //! @brief Get a pointer to the content
    void* Content() const { return m_pContent; }

    //! The width of the bitmap in pixels
    int32_t Width() const { return m_width; }

    //! The height of the bitmap in pixels
    int32_t Height() const { return m_height; }

    //! The number of bytes that represent one scan line of the bitmap.
    int32_t Stride() const { return m_stride; }

    //! The number of bytes that can be stored in m_pContent
    int32_t GetBufferLength() const { return m_height * m_stride; }

    //! Get the pixel format of the raw bitmap
    PixelFormat::Enum GetPixelFormat() const { return m_pixelFormat; }

    //! Conversion operator for easy conversion to a RawBitmap
    operator RawBitmap() const { return RawBitmap(m_pContent, m_width, m_height, m_pixelFormat, m_stride); }
  };
}

#endif


