#ifndef FSLGRAPHICS_RAWBITMAPUTIL_HPP
#define FSLGRAPHICS_RAWBITMAPUTIL_HPP
/****************************************************************************************************************************************************
* Copyright (c) 2014 Freescale Semiconductor, Inc.
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*
*    * Redistributions of source code must retain the above copyright notice,
*      this list of conditions and the following disclaimer.
*
*    * Redistributions in binary form must reproduce the above copyright notice,
*      this list of conditions and the following disclaimer in the documentation
*      and/or other materials provided with the distribution.
*
*    * Neither the name of the Freescale Semiconductor, Inc. nor the names of
*      its contributors may be used to endorse or promote products derived from
*      this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
* ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
****************************************************************************************************************************************************/

#include <FslGraphics/RawBitmap.hpp>

namespace Fsl
{
  class RawBitmapEx;

  //! @brief Contains helper methods for working on bitmaps
  //         They exist to provide functionality but they are not performance optimized!
  class RawBitmapUtil
  {
  public:
    //! @brief Given a raw bitmap calculate the row alignment that fits the stride or throw a UnsupportedStrideException if
    //         none fits (the alignment is the minimum padding necessary after a pixel row to start the next one at the given alignment).
    static int CalcAlignment(const RawBitmap& rawBitmap, const int bytesPerPixel);

    // @brief Swizzle the color channels in a 24 or 32bpp image according to the supplied rules
    // @param rBitmap the bitmap to swizzle.
    // @param srcIdx0 the location to read 'byte0' from (0-3 32bpp) (0-2 24bpp)
    // @param srcIdx1 the location to read 'byte1' from (0-3 32bpp) (0-2 24bpp)
    // @param srcIdx2 the location to read 'byte2' from (0-3 32bpp) (0-2 24bpp)
    // @param srcIdx3 the location to read 'byte3' from (0-3 32bpp) (ignored for 24bpp)
    // @note Only RGB888, BGR888, RGBA8888 and BGRA8888 formats are supported at the moment, throws UnsupportedPixelFormatException if the pixel format is unsupported
    static void Swizzle(RawBitmapEx& rBitmap, const int srcIdx0, const int srcIdx1, const int srcIdx2, const int srcIdx3);

    // @brief Swizzle the color channels in a 24bpp image according to the supplied rules
    // @param rBitmap the bitmap to swizzle.
    // @param srcIdx0 the location to read 'byte0' from (0-2)
    // @param srcIdx1 the location to read 'byte1' from (0-2)
    // @param srcIdx2 the location to read 'byte2' from (0-2)
    // @note Only RGBA8888 and BGRA8888 formats are supported at the moment, throws UnsupportedPixelFormatException if the pixel format is unsupported
    static void Swizzle24(RawBitmapEx& rBitmap, const int srcIdx0, const int srcIdx1, const int srcIdx2);

    // @brief Swizzle the color channels in a 32bpp image according to the supplied rules
    // @param rBitmap the bitmap to swizzle.
    // @param srcIdx0 the location to read 'byte0' from (0-3)
    // @param srcIdx1 the location to read 'byte1' from (0-3)
    // @param srcIdx2 the location to read 'byte2' from (0-3)
    // @param srcIdx3 the location to read 'byte3' from (0-3)
    // @note Only RGBA8888 and BGRA8888 formats are supported at the moment, throws UnsupportedPixelFormatException if the pixel format is unsupported
    static void Swizzle32(RawBitmapEx& rBitmap, const int srcIdx0, const int srcIdx1, const int srcIdx2, const int srcIdx3);

    //! @brief Zero fill the padding area if any exist (the padding area is the area minimum stride that is possible and stride that is being used)
    static void ClearPadding(RawBitmapEx& rBitmap);

    //! @brief Flip the bitmap about the y-axis
    static void FlipHorizontal(RawBitmapEx& rBitmap);
  };
}

#endif


