#ifndef FSLGRAPHICS_VERTICES_VERTEXDECLARATION_HPP
#define FSLGRAPHICS_VERTICES_VERTEXDECLARATION_HPP
/****************************************************************************************************************************************************
* Copyright (c) 2014 Freescale Semiconductor, Inc.
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*
*    * Redistributions of source code must retain the above copyright notice,
*      this list of conditions and the following disclaimer.
*
*    * Redistributions in binary form must reproduce the above copyright notice,
*      this list of conditions and the following disclaimer in the documentation
*      and/or other materials provided with the distribution.
*
*    * Neither the name of the Freescale Semiconductor, Inc. nor the names of
*      its contributors may be used to endorse or promote products derived from
*      this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
* ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
****************************************************************************************************************************************************/

#include <FslGraphics/Vertices/VertexElement.hpp>
#include <FslGraphics/Vertices/VertexElementEx.hpp>
#include <vector>

namespace Fsl
{
  class VertexDeclaration
  {
    std::vector<VertexElementEx> m_elements;
    int m_vertexStride;
  public:
    VertexDeclaration();

    //! @brief Create a vertex declaration based on the given elements.
    VertexDeclaration(const VertexElement*const pElements, const int elementCount);

    //! @brief Create a vertex declaration based on the given elements.
    //! @note  Beware that the elements will be force sorted according to offset (smallest to largest)
    VertexDeclaration(const VertexElementEx*const pElements, const int elementCount, const int vertexStride);

    //! @brief Create a vertex declaration based on the given elements.
    void Reset(const VertexElement*const pElements, const int elementCount);

    //! @brief Create a vertex declaration based on the given elements.
    //! @note  Beware that the elements will be force sorted according to offset (smallest to largest)
    void Reset(const VertexElementEx*const pElements, const int elementCount, const int vertexStride);

    //! @brief Get the vertex stride
    int VertexStride() const { return m_vertexStride; }

    //! @brief Get the number of elements
    int Count() const;

    //! @brief Get direct access to the elements
    const VertexElementEx* DirectAccess() const;

    //! @brief Find the element index of for the given usage and usageIndex (if not found <0 is returned)
    int VertexElementIndexOf(const VertexElementUsage::Enum usage, const int usageIndex) const;
  };
}

#endif


