package com.freescale.contentsync;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;

public class Cache {

	private static final String HEADER = "# FslContentSync cache file";
	private static final String HEADER_FORMAT = "# Format: 1";
	
	public ArrayList<CacheEntry> Files = new ArrayList<CacheEntry>();
	public ArrayList<CacheEntry> Dirs = new ArrayList<CacheEntry>();

	public Cache()
	{
	}
	
	public Cache(InputStream stream) throws UnsupportedEncodingException,
			IOException {
		String content = FileUtil.LoadUTF8(stream);
		Prepare(content);
	}

	public Cache(String filename) throws UnsupportedEncodingException,
			IOException {
		try
		{
			String content = FileUtil.LoadUTF8(filename);
			Prepare(content);
		}
		catch(FileNotFoundException ex)
		{
			
		}
	}

	private boolean ValidateHeader(String[] lines){
		if(lines.length < 2 )
			return false;
		if( ! lines[0].equals(HEADER) ) {
			return false;
		}
		if( ! lines[1].equals(HEADER_FORMAT) ) {
			return false;
		}
		return true;
	}
	
	private void Prepare(String content) throws UnsupportedEncodingException,
			IOException {
		String[] lines = content.split("\n");

		if( ! ValidateHeader(lines) )
			return;
		
		for (int i = 0; i < lines.length; i++) {
			String line = lines[i];
			if (!line.startsWith("#")) {
				String[] parts = line.split("\\\\");
				if (parts.length != 4)
					throw new IOException("Cache entry is invalid");

				long length = Long.parseLong(parts[1]);
				CacheEntry entry = new CacheEntry(parts[0], length, parts[2],
						parts[3]);

				if (length >= 0)
					Files.add(entry);
				else
					Dirs.add(entry);
			}
		}
	}

}
